//*****************************************************************************
//* OpenLog DRIVER FOR PIC32MX595F512H
//* WRITTEN BY FRED EADY
//* NO USB SUPPORT IN THIS VERSION
//* MPLABX 3.05 XC32 1.34
//* LAST UPDATE	08/17/2015
//* NOTES:
//*****************************************************************************

//*****************************************************************************
//* CONFIGURATION BITS
//*****************************************************************************
    #pragma config UPLLEN   = OFF           // USB PLL Disabled
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    //#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
//*****************************************************************************
//* Internal Function Prototypes
//*****************************************************************************
#define _SUPPRESS_PLIB_WARNING
#include <xc.h>
#include <plib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <GenericTypeDefs.h>
#include "portMacros.h"
#include "microsd_Driver.h"
//*****************************************************************************
//* USART RECEIVE BUFFERS SETUP
//* 1,2,4,8,16,32,64,128 or 256 bytes 
//*****************************************************************************
#define USART1_RX_BUFFER_SIZE  256      
#define USART1_RX_BUFFER_MASK ( USART1_RX_BUFFER_SIZE - 1 )
BYTE USART1_RxBuf[USART1_RX_BUFFER_SIZE];
BYTE USART1_RxTail;
BYTE USART1_RxHead;

#define USART2_RX_BUFFER_SIZE  256      
#define USART2_RX_BUFFER_MASK ( USART2_RX_BUFFER_SIZE - 1 )
BYTE USART2_RxBuf[USART2_RX_BUFFER_SIZE];
BYTE USART2_RxTail;
BYTE USART2_RxHead;

#define USART4_RX_BUFFER_SIZE  256      
#define USART4_RX_BUFFER_MASK ( USART4_RX_BUFFER_SIZE - 1 )
BYTE USART4_RxBuf[USART4_RX_BUFFER_SIZE];
BYTE USART4_RxTail;
BYTE USART4_RxHead;

#define USART5_RX_BUFFER_SIZE  256      
#define USART5_RX_BUFFER_MASK ( USART5_RX_BUFFER_SIZE - 1 )
BYTE USART5_RxBuf[USART5_RX_BUFFER_SIZE];
BYTE USART5_RxTail;
BYTE USART5_RxHead;
//*****************************************************************************
//* SRAM ALLOCATIONS
//*****************************************************************************
BYTE scratch8;
BYTE bitein;
//char rxBuf5[256];
//char rxBuf2[256];
//char rxBuf4[256];
//char indx5,indx2,indx4;
//char fileName[16];

const BYTE atmelPrompt[] = "12>";
const BYTE atmelError[] = "!";
BYTE rxBufAtmel[1024];
BYTE rxBufProg[1024];
WORD idxBufAtmel;
//*****************************************************************************
// UART 1 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_1_VECTOR, IPL2SOFT) IntUart1Handler(void)
{
		BYTE data,tmphead,tmptail;
		
		data = U1RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART1_RxHead + 1 ) & USART1_RX_BUFFER_MASK;
		USART1_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART1_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART1_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS0CLR = 0x08000000;//0000 1000 0000 0000 0000 0000 0000 0000
}
//*****************************************************************************
// UART 2 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_2_VECTOR, IPL2SOFT) IntUart2Handler(void)
{
		BYTE data,tmphead,tmptail;
		
		data = U2RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART2_RxHead + 1 ) & USART2_RX_BUFFER_MASK;
		USART2_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART2_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
                                
		USART2_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS1CLR = 0x00000200;//0000 0000 0000 0000 0000 0010 0000 0000
}
//*****************************************************************************
// UART 4 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_4_VECTOR, IPL2SOFT) IntUart4Handler(void)
{
BYTE data,tmphead,tmptail;
		
		data = U4RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART4_RxHead + 1 ) & USART4_RX_BUFFER_MASK;
		USART4_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART4_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART4_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS2CLR = 0x00000010;
}
//*****************************************************************************
// UART 5 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_5_VECTOR, IPL2SOFT) IntUart5Handler(void)
{
BYTE data,tmphead,tmptail;
		
		data = U5RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART5_RxHead + 1 ) & USART5_RX_BUFFER_MASK;
		USART5_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART5_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART5_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS2CLR = 0x00000400; //0000 0000 0000 0000 0000 0100 0000 0000
}
//******************************************************************
//*	USART1 Receive Character Function
//******************************************************************
BYTE recvchar1(void)
{
  BYTE tmptail1;
  // wait for incoming data 
  while ( USART1_RxHead == USART1_RxTail );
  // calculate buffer index 
  tmptail1 = ( USART1_RxTail + 1 ) & USART1_RX_BUFFER_MASK;
  // store new index 
  USART1_RxTail = tmptail1;
  return USART1_RxBuf[tmptail1];
}
//******************************************************************
//*	USART2 Receive Character Function
//******************************************************************
BYTE recvchar2(void)
{
  BYTE tmptail2;
  // wait for incoming data 
  while ( USART2_RxHead == USART2_RxTail );
  // calculate buffer index 
  tmptail2 = ( USART2_RxTail + 1 ) & USART2_RX_BUFFER_MASK;
  // store new index 
  USART2_RxTail = tmptail2;
  return USART2_RxBuf[tmptail2];
}
//******************************************************************
//*	USART4 Receive Character Function
//******************************************************************
BYTE recvchar4(void)
{
  BYTE tmptail4;
  // wait for incomming data
  while ( USART4_RxHead == USART4_RxTail );
  // calculate buffer index
  tmptail4 = ( USART4_RxTail + 1 ) & USART5_RX_BUFFER_MASK;
  // store new index
  USART4_RxTail = tmptail4;
  return USART4_RxBuf[tmptail4];
}
//******************************************************************
//*	USART5 Receive Character Function
//******************************************************************
BYTE recvchar5(void)
{
  BYTE tmptail5;
  // wait for incomming data
  while ( USART5_RxHead == USART5_RxTail );
  // calculate buffer index
  tmptail5 = ( USART5_RxTail + 1 ) & USART5_RX_BUFFER_MASK;
  // store new index
  USART5_RxTail = tmptail5;
  return USART5_RxBuf[tmptail5];
}
//******************************************************************
//*	USART1 Character Waiting Function
//******************************************************************
BYTE CharInQueue1(void)
{
  return(USART1_RxHead != USART1_RxTail);
}
//******************************************************************
//*	USART2 Character Waiting Function
//******************************************************************
BYTE CharInQueue2(void)
{
  return(USART2_RxHead != USART2_RxTail);
}
//******************************************************************
//*	USART4 Character Waiting Function
//******************************************************************
BYTE CharInQueue4(void)
{
  return(USART4_RxHead != USART4_RxTail);
}
//******************************************************************
//*	USART5 Character Waiting Function
//******************************************************************
BYTE CharInQueue5(void)
{
  return(USART5_RxHead != USART5_RxTail);
}
//******************************************************************************
//* UART1 REDIRECTOR FOR USB PORTAL
//******************************************************************************
void _mon_putc(char c)
{
	U1TXREG = c;
	while(U1STAbits.TRMT == 0);
}
//******************************************************************************
//* CREATE NEW FILE
//******************************************************************************
void newFile(BYTE filenum)
{
    BYTE bitein;
    char cmdBuf[32];
    
    //send new file command
    sprintf(cmdBuf,"new microsd%1d.csv\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for command mode prompt
    do{
    if(CharInQueue2())
    {
        bitein = recvchar2();
    }
    }while(bitein != '>');
}
//******************************************************************************
//* WRITE File
//******************************************************************************
void writeFile(BYTE filenum)
{
    char cmdBuf[32];
    BYTE bitein;
    
    //send write command
    sprintf(cmdBuf,"write microsd%1d.csv 0\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for card to indicate file is open and ready for writing
    do{
        if(CharInQueue2())
        {
            bitein = recvchar2();
        }
      }while(bitein != '<');
      
    //put data to write here
    sendBiteUart2('A');
    sendBiteUart2(',');
    sendBiteUart2('B');
    sendBiteUart2(',');
    sendBiteUart2('C');
    sendBiteUart2(',');
    sendBiteUart2(0x01);
    //commit the data to the card
    sendBiteUart2(0x0D);
    //send an empty line to end the write
    sendBiteUart2(0x0D);
    //wait for command mode prompt
        do{
        if(CharInQueue2())
        {
            bitein = recvchar2();
        }
    }while(bitein != '>');
}
//******************************************************************************
//* GET FILE SIZE
//******************************************************************************
WORD getFileSize(BYTE fileNum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i,j;
    WORD fileSize;
    char sizeBuf[8];

    memset(sizeBuf,0x00,sizeof(sizeBuf));
   
    //send size command
    sprintf(cmdBuf,"size microsd%1d.csv\r",fileNum);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
            rxBufProg[i++] = recvchar2();
        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufProg[i-1] != '>'));
      i = 0;
      j = 0;
      	do{
		if(isspace(rxBufProg[j]))
		{
			++j;
		}
	}while(isspace(rxBufProg[j]));

      do{
          sizeBuf[i++] = rxBufProg[j++];
      }while(rxBufProg[j-1] != 0x0D && rxBufProg[j-1] != 0x0A);
      sizeBuf[i-1] = 0x00;
      fileSize = atoi(sizeBuf);
      return(fileSize);
 }
//******************************************************************************
//* READ File
//******************************************************************************
void readFile(BYTE fileNum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    WORD fileSize;
    BYTE bitein;
   
    fileSize = getFileSize(fileNum);
    
    memset(rxBufProg,0x00,sizeof(rxBufProg));
    //send read command
    sprintf(cmdBuf,"read microsd%1d.csv 0 %u 3\r",fileNum,fileSize);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
        	bitein = recvchar2();
            sendBiteUart1(bitein);
            rxBufProg[i++] = bitein;
        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufProg[i-1] != '>'));
 }
//******************************************************************************
//* READ File - ASCII only
//******************************************************************************
void readFileAscii(BYTE filenum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    BYTE bitein;
    
    memset(rxBufAtmel,0x00,sizeof(rxBufAtmel));
    //send read command
    sprintf(cmdBuf,"read microsd%1d.csv\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
        	bitein = recvchar2();
            sendBiteUart1(bitein);
            rxBufAtmel[i++] = bitein;

        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufAtmel[i-1] != '>'));
 }
//******************************************************************************
//* LIST File
//******************************************************************************
void listFile(void)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    BYTE bitein;
    
    //send list file command
    sprintf(cmdBuf,"ls\r");
    sendStrUart2(cmdBuf);
    //wait for card to respond to ls 
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
            do{
            bitein = recvchar2();
            sendBiteUart1(bitein);
            }while(CharInQueue2());
        }
        tdelayms(1);
      }while(timeoutms-- && (bitein != '>'));
 }
//*****************************************************************************
//* IDLE FUNCTION
//*****************************************************************************
void idle(void)
{
    BYTE bitein;
    
	if(CharInQueue1())
	{
        bitein = recvchar1();
        printf("%c",bitein);
 	}
}
//******************************************************************************
//* INITIALIZE FUNCTION
//******************************************************************************
void init(void)
 {
    WORD  value;
    WORD i;
    WORD timeoutms;
    
    AD1PCFG = 0xFFFF;
    value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

    // Enable the cache for the best performance
    CheKseg0CacheOn();
	// Wait for PLL lock to stabilize
    value = OSCCON;
    while (!(value & 0x00000020))
    {
    value = OSCCON;    
    }
    
    DDPCONbits.JTAGEN = 0;

    TRISBCLR = 0xC020; //1100 0000 0010 0000
    TRISDCLR = 0x000A; //0000 0000 0000 1010
    TRISECLR = 0x003C; //0000 0000 0011 1100
    TRISFCLR = 0x0020; //0000 0000 0010 0000
	
    gpio9LO;
    bluLED_Off;
    redLED_Off;
    grnLED_Off;
   


    rstLO;
    tdelayms(100);
    rstHI;
    tdelayms(5000);

    
    //Initialize UART1
    //U1BRG = 43; // Set Baud rate 115200
    U1BRG = 520; // Set Baud rate 9600
    U1STA = 0;
    U1MODE = 0x00008000; // Enable UART for 8-none-1
    U1STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC6SET = 0x0000000B;
    IEC0SET = 0x08000000;
    IFS0CLR = 0x08000000;
    // flush receive buffer 
    USART1_RxTail = 0x00;
    USART1_RxHead = 0x00;

    //Initialize UART2
    U2BRG = 520; // Set Baud rate 9600
    U2STA = 0;
    U2MODE = 0x00008000; // Enable UART for 8-none-1    
    U2STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC8SET = 0x0000000B;
    IEC1SET = 0x00000200;
    IFS1CLR = 0x00000200;
    // flush receive buffer 
    USART2_RxTail = 0x00;
    USART2_RxHead = 0x00;
    
    //Initialize UART4
    U4BRG = 520; // Set Baud rate 9600
    U4STA = 0;
    U4MODE = 0x00008000; // Enable UART for 8-none-1
    U4STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <12:10> sub bits <9:8>
    IPC12SET = 0x00000B00;
    IEC2SET = 0x00000010;
    IFS2CLR = 0x00000010;
    //flush receive buffer 
    USART4_RxTail = 0x00;
    USART4_RxHead = 0x00;

    //Initialize UART5
    U5BRG = 43; // Set Baud rate 115200
    U5STA = 0;
    U5MODESET = 0x00008000; // disable UART for 8-none-1
    U5STASET = 0x00001400; // Enable Transmit and Receive
     //priority 2 sub priority 3 - 00011111 bits <26:28> sub bits <25:24>
    IPC12SET = 0x0B000000;//0000 1011 0000 0000 0000 0000 0000 0000
    IEC2SET = 0x00000400;
    IFS2CLR = 0x00000400;
    //flush receive buffer 
    USART5_RxTail = 0x00;
    USART5_RxHead = 0x00;

    USART1_RxHead = 0;
    USART1_RxTail = 0;
    USART2_RxHead = 0;
    USART2_RxTail = 0;
    USART4_RxHead = 0;
    USART4_RxTail = 0;
    USART5_RxHead = 0;
    USART5_RxTail = 0;

    //Enable Interrupts
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    ei();
    tdelayms(10);
    atmrstLO;
    bluLED_On;
    redLED_On;
    tdelayms(100);
    atmrstHI;
    i=0;
    timeoutms = 5000;
    do{
    if(CharInQueue2())
    {
         do{
            rxBufAtmel[i++] = recvchar2();
         }while(CharInQueue2());
    }
             tdelayms(1);

  }while(--timeoutms && strcmp(atmelPrompt,rxBufAtmel) != 0);
  
  if(timeoutms)
  {
	 for(i=0;i<15;++i)
	 {
		 bluLED_Tog;
		 tdelayms(25);
		 redLED_Tog;
		 tdelayms(25);
	 }
	 bluLED_Off;
	 redLED_Off;
 }
 else
 {  
	 di();
	 bluLED_Off;
	 redLED_Off;
	 do{
		 bluLED_Tog;
		 redLED_Tog;
		 tdelayms(100);
	 }while(1);
  }
}
//******************************************************************************
// MAIN
//******************************************************************************
int main (void)
{
    BYTE bitein;
	init();
	newFile(1);
	listFile();
	writeFile(1);
    readFile(1);
    listFile();
	do{
		bluLED_Tog;
		tdelayms(250);
	}while(1);
/*
//****************************
// TERMINAL INTERFACE
//****************************
    do{
	if(CharInQueue1())
	{
            do{
            bitein = recvchar1();
            sendBiteUart2(bitein);
            }while(CharInQueue1());
	}
	if(CharInQueue2())
	{
            do{
            bitein = recvchar2();
            sendBiteUart1(bitein);
            }while(CharInQueue2());
	}
    }while(1);
*/
}



